import * as Amount from "./amount";
/**
 * @module Serialize
 */
/**
 * Converts a units serialized representation to it's deserialized representation
 * @param unitString
 */
export function stringToUnit(unitString, units) {
    return units[unitString];
}
/**
 * Converts a unit to it's serialized representation
 * @param unit
 */
export function unitToString(unit) {
    return unit.name;
}
/**
 * Convert an amount to it's serialized representation
 * @param amount
 */
export function amountToString(amount) {
    if (!amount.value === null || amount.value === undefined) {
        return "";
    }
    var valueString = amount.value.toFixed(amount.decimalCount);
    var unitString = unitToString(amount.unit);
    return valueString + ":" + unitString;
}
/**
 * Convert a serialized amount to it's deserialized representation
 * @param amountString
 */
export function stringToAmount(amountString, units) {
    var parts = amountString.split(":");
    var value = parseFloat(parts[0]);
    var unit = stringToUnit(parts[1], units);
    if (!unit) {
        return undefined;
    }
    var decimalCount = 0;
    var pointIndex = parts[0].indexOf(".");
    if (pointIndex >= 0) {
        decimalCount = parts[0].length - pointIndex - 1;
    }
    return Amount.create(value, unit, decimalCount);
}
//# sourceMappingURL=serialize.js.map